//
//  LinearProgressLayer3.m
//  CALayerAnimationsExample
//
//  Created by Matthew Houlden on 27/01/2017.
//  Copyright © 2017 Airsource. All rights reserved.
//

#import "LinearProgressLayer3.h"

@implementation LinearProgressLayer3

@dynamic progress;

- (id)init
{
    self = [super init];
    if (self)
    {
        self.trackTintColor = [UIColor lightGrayColor];
        self.progressTintColor = [UIColor blueColor];
        [self setNeedsDisplay];
    }
    return self;
}

- (id)initWithLayer:(id)layer
{
    self = [super initWithLayer:layer];
    if (self)
    {
        if ([layer isKindOfClass:[LinearProgressLayer3 class]])
        {
            LinearProgressLayer3 *other = (LinearProgressLayer3 *)layer;
            self.trackTintColor = other.trackTintColor;
            self.progressTintColor = other.progressTintColor;
        }
    }
    return self;
}

+ (BOOL)needsDisplayForKey:(NSString *)key
{
    if ([key isEqualToString:@"progress"])
    {
        return YES;
    }
    return [super needsDisplayForKey:key];
}

- (id<CAAction>)actionForKey:(NSString *)event
{
    if ([event isEqualToString:@"progress"])
    {
        CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:event];
        animation.fromValue = [[self presentationLayer] valueForKey:event];
        animation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
        animation.duration = 1.f;
        return animation;
    }
    return [super actionForKey:event];
}

- (void)drawInContext:(CGContextRef)context
{
    [super drawInContext:context];
    
    // Draw the track bar
    CGPathRef path1 = [self pathForProgress:100.f];
    CGContextAddPath(context, path1);
    CGPathRelease(path1);
    CGContextSetFillColorWithColor(context, self.trackTintColor.CGColor);
    CGContextDrawPath(context, kCGPathFill);
    
    // Draw the progress bar
    CGPathRef path2 = [self pathForProgress:self.progress];
    CGContextAddPath(context, path2);
    CGPathRelease(path2);
    CGContextSetFillColorWithColor(context, self.progressTintColor.CGColor);
    CGContextDrawPath(context, kCGPathFill);
}

- (CGMutablePathRef)pathForProgress:(CGFloat)progress
{
    CGFloat height = self.frame.size.height;
    CGFloat radius = height / 2;
    CGFloat xMin = radius;
    CGFloat xMax = self.frame.size.width - radius;
    CGFloat x = xMin + (progress / 100.f) * (xMax - xMin);
    
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathMoveToPoint(path, NULL, xMin, radius * 2);
    CGPathAddArc(path, NULL, xMin, radius, radius, M_PI / 2, 3 * M_PI / 2, NO);
    CGPathAddLineToPoint(path, NULL, x, 0);
    CGPathAddArc(path, NULL, x, radius, radius, 3 *M_PI / 2, M_PI / 2, NO);
    return path;
}

@end
